/*
 GAMP - (c) 2001 - Renato R. Santana <rrsantana@yahoo.com>
 
 This file contains file related functions.
   
 This file is part of GAMP, the GNU Amiga Module Player
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
    
 You should have received a copy of the GNU General Public License
 along with This program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "mod.h"

struct modpattern				patterns[128];
extern struct modheadertype		mod_header;
char							*instr_body_ptr[MAXINS];
int								mp_hipatt;

unsigned int big_endian_to_int(unsigned short falseword) {
	unsigned int ret;
	
	ret=(falseword & 0xff00) >> 8;
	ret=(ret + ((falseword & 0x00ff) << 8));
	return(ret);
}

int mp_load(char *modfilename){
	
	int						fd;
	FILE 					*modfile;
	unsigned short			result;
	char 					dummystr[80];
	int						inscount;
	int						patcount;
	int						counter;
	int						n = 0;
	int						i = 0;
	int						ret_code = 0;

	/* Make sure that desired MOD file is available */
	if (!(modfile = fopen(modfilename, "r"))) {
	   printf("Error while opening modfile.\n");
	   ret_code = 1;
	   goto mp_load_exit;
	}

	/* Read MOD header data into header structure */
	result = fread(&mod_header, sizeof(mod_header), 1, modfile);

	if (strncmp("M.K.", mod_header.modmark, 4) != 0 &&
		strncmp("M!K!", mod_header.modmark, 4) != 0 &&
		strncmp("FLT4", mod_header.modmark, 4) != 0 &&
		strncmp("4CHN", mod_header.modmark, 4) != 0 ) {
		printf("Not suported or not recognized file format.\n");
		ret_code = 1;
		goto mp_load_exit;
	}
	
	/* convert big endian values to int */
	for (i = 0; i <= MAXINS - 1; i++) {
		mod_header.modinstr[i].samplen = big_endian_to_int(mod_header.modinstr[i].samplen) * 2;
		mod_header.modinstr[i].sampreps = big_endian_to_int(mod_header.modinstr[i].sampreps) * 2;
		mod_header.modinstr[i].samprepl	= big_endian_to_int(mod_header.modinstr[i].samprepl) * 2;
	}

	/* Search for highest pattern number */
	mp_hipatt = 0;
	for (patcount = 0; patcount<=mod_header.modlen - 1; patcount++)
		if (mod_header.modarrangement[patcount] >= mp_hipatt)
            mp_hipatt = mod_header.modarrangement[patcount];
	

	/* load all patterns data at once */ 
	result = fread(&patterns, (mp_hipatt + 1) * 1024, 1, modfile);

	/* now load instruments bodies */
	for (i = 0; i <= MAXINS - 1; i++){
		if (mod_header.modinstr[i].samplen > 2){
			if ( (instr_body_ptr[i] = (char *)malloc(mod_header.modinstr[i].samplen)) == 0){
				printf("Error allocating memory for instrument data.\n");
				ret_code = 1;
				goto mp_load_exit;
			}
			result = fread(instr_body_ptr[i], mod_header.modinstr[i].samplen, 1, modfile);
			if (result == 0){
				printf("Error loading instrument data.\n");
				ret_code = 1;
				goto mp_load_exit;
			}
		}
	}
   
	/* check for instrument data anomalies */
	for (i = 0; i <= MAXINS - 1; i++){
		if (mod_header.modinstr[i].samplen > 2) {
			mod_header.modinstr[i].sampreps
			= mod_header.modinstr[i].sampreps
		   	> mod_header.modinstr[i].samplen
			? 0 : mod_header.modinstr[i].sampreps;

			mod_header.modinstr[i].samprepl
			= mod_header.modinstr[i].samprepl
		   	> mod_header.modinstr[i].samplen - mod_header.modinstr[i].sampreps + 2
			? 0 : mod_header.modinstr[i].samprepl;
		}
	}

mp_load_exit:
	if (modfile)
		fclose(modfile);
	return(ret_code);
}

int mp_unload(void){
	int 		i;	
	
	for (i = 0; i <= MAXINS - 1; i++)
		if (mod_header.modinstr[i].samplen > 2) 
			free( instr_body_ptr[i] );
}

